@@ -10,8 +10,8 @@ from enum import Enum
 import itertools
 from typing import Union, Tuple, List
 import matplotlib.pyplot as plt
-import openseespy.opensees as ops
-import opsvis as opsv
+import openseespy.opensees as osp
+import opsvis as ospv
 import numpy as np
 from .utils import save_figs
 
@@ -92,7 +92,7 @@ class Node:
         Parameters
         ----------
         support : Support
-            The support type.
+            The support type, a :class:`ospgrid.grid.Support` object
 
         Returns
         -------
@@ -134,25 +134,27 @@ class Member:
         self.node_j = node_j
         self.EI = EI
         self.GJ = GJ
+        self.delta_x = self.node_j.x - self.node_i.x
+        self.delta_y = self.node_j.y - self.node_i.y
+        self.L = np.sqrt(self.delta_x ** 2 + self.delta_y ** 2)
 
     def get_local_stiffness(self) -> np.ndarray:
         """
-        Gives the element stiffness matrix in local member coordinates with DOFs
+        Returns the member stiffness matrix in local coordinates with nodal DOFs
         in the order DZ ('vertical' force), RX (torsion), RZ (bending moment).
 
         Returns
         -------
         K : np.ndarray
-            Element stiffness matrix in local coordinates.
+            Member stiffness matrix in local coordinates.
 
         """
-        L = (self.node_i.x - self.node_j.x) ** 2 + (self.node_i.y - self.node_j.y) ** 2
-        L = np.sqrt(L)
-        k11 = 12 * self.EI / L**3
-        k13 = 6 * self.EI / L**2
-        k22 = self.GJ / L
-        k33 = 4 * self.EI / L
-        k36 = 2 * self.EI / L
+
+        k11 = 12 * self.EI / self.L ** 3
+        k13 = 6 * self.EI / self.L ** 2
+        k22 = self.GJ / self.L
+        k33 = 4 * self.EI / self.L
+        k36 = 2 * self.EI / self.L
 
         K = np.array(
             [
@@ -166,6 +168,43 @@ class Member:
         )
         return K
 
+    def get_transformation_matrix(self) -> np.ndarray:
+        """
+        Gives the transformation matrix relating the member dgrees of freedom from
+        local to global coordinate system.
+
+        It considers only the grid member DOFs, per :func:`get_local_stiffness`
+
+        Returns
+        -------
+        T : np.ndarray
+            Tranformation matrix
+
+        """
+        c = self.delta_x / self.L
+        s = self.delta_y / self.L
+
+        T = np.kron(np.eye(2, dtype=int), np.array([[1, 0, 0], [0, c, s], [0, -s, c]]))
+        return T
+
+    def get_global_stiffness(self) -> np.ndarray:
+        """
+        Returns the member stiffness matrix in global coordinates with nodal DOFs
+        in the order DZ ('vertical' force), RX (torsion), RZ (bending moment). This
+        shows the stiffness
+        contributions of the member to the global DOFs.
+
+        Returns
+        -------
+        Kg : np.ndarray
+            Member stiffness matrix in global coordinates.
+
+        """
+        K = self.get_local_stiffness()
+        T = self.get_transformation_matrix()
+        Kg = T.T @ K @ T
+        return Kg
+
 
 class Grid:
     """
@@ -253,8 +292,8 @@ class Grid:
 
         """
         self.no_members += 1
-        the_node_i = self._get_node(node_i)
-        the_node_j = self._get_node(node_j)
+        the_node_i = self.get_node(node_i)
+        the_node_j = self.get_node(node_j)
         member = Member(self.no_members, the_node_i, the_node_j, EI, GJ)
         self.members.append(member)
         return member
@@ -291,10 +330,12 @@ class Grid:
         None.
 
         """
-        the_node = self._get_node(node)
+        the_node = self.get_node(node)
         the_node.set_load(Fz, Mx, My)
 
-    def add_support(self, node: Union[Node, str, int], support: Support = None):
+    def add_support(
+        self, node: Union[Node, str, int], support: Union[Support, str] = None
+    ):
         """
         Add a support to a node in the grid.
 
@@ -302,18 +343,38 @@ class Grid:
         ----------
         node : Union[Node,str,int]
             Node object, label, or id
-        support : Support
-            The support type for the node.
+        support : Union[Support,str]
+            The support type, a :class:`ospgrid.grid.Support` object. Alternatively,
+            a single character support descriptor as a :class:`str` can be used as
+            follows:
+
+                - `X` = :attr:`ospgrid.grid.Support.PINNED_X`
+                - `Y` = :attr:`ospgrid.grid.Support.PINNED_Y`
+                - `F` = :attr:`ospgrid.grid.Support.FIXED`
+                - `V` = :attr:`ospgrid.grid.Support.FIXED_V_ROLLER`
+                - `P` = :attr:`ospgrid.grid.Support.PROP`
 
         Returns
         -------
         None.
 
         """
-        the_node = self._get_node(node)
-        the_node.set_support(support)
+        the_node = self.get_node(node)
+        if isinstance(support, str):
+            if support == "F":
+                the_node.set_support(Support.FIXED)
+            elif support == "X":
+                the_node.set_support(Support.PINNED_X)
+            elif support == "Y":
+                the_node.set_support(Support.PINNED_Y)
+            elif support == "P":
+                the_node.set_support(Support.PROP)
+            elif support == "V":
+                the_node.set_support(Support.FIXED_V_ROLLER)
+        else:
+            the_node.set_support(support)
 
-    def _get_node(self, node=Union[Node, str, int]):
+    def get_node(self, node=Union[Node, str, int]):
         """
         Gets the node from an node object, id or label.
 
@@ -351,7 +412,7 @@ class Grid:
         # We should never get here
         raise ValueError("Either node object, label, or node id must be passed")
 
-    def _get_member(self, member=Union[Member, int, Tuple[str, str]]):
+    def get_member(self, member=Union[Member, int, Tuple[str, str]]):
         """
         Gets the member from a member object, id, or a tuple of node labels.
 
@@ -398,39 +459,39 @@ class Grid:
         # We should never get here
         raise ValueError("Either member object or id must be passed")
 
-    def analyze(self):
+    def analyze(self) -> osp:
         """
         Executes the analysis for the grid object using OpenSeesPy
 
         Returns
         -------
-        ops : OpenSeesPy instance
+        osp : OpenSeesPy instance
             The OpenSeesPy instance, which can be used for querying results directly
             or otherwise manipulating the model further.
 
         """
         # remove any existing model
-        ops.wipe()
+        osp.wipe()
 
         # set modelbuilder, 3 dims, 6 DOF
-        ops.model("basic", "-ndm", 3, "-ndf", 6)
+        osp.model("basic", "-ndm", 3, "-ndf", 6)
 
         # create nodes & add support, load, if any
         for n in self.nodes:
-            ops.node(n.idx, n.x, n.y, 0.0)
+            osp.node(n.idx, n.x, n.y, 0.0)
             if n.support is not None:
-                ops.fix(n.idx, *n.support.value)
+                osp.fix(n.idx, *n.support.value)
 
         # Nominal E and G
         E = 200e9  # GPa
         G = 80e9  # GPa
 
         # define materials
-        ops.uniaxialMaterial("Elastic", 1, E)
+        osp.uniaxialMaterial("Elastic", 1, E)
 
         # Define geometry transforms
         vecxz = [0, 0, 1]
-        ops.geomTransf("Linear", 1, *vecxz)
+        osp.geomTransf("Linear", 1, *vecxz)
 
         # define elements
         # tag   *[ndI ndJ]  A  E  G  Jx  Iy   Iz  transfOBJs
@@ -438,7 +499,7 @@ class Grid:
             I = m.EI / E
             J = m.GJ / G
             A = I / 1e6  # rough
-            ops.element(
+            osp.element(
                 "elasticBeamColumn",
                 m.idx,
                 m.node_i.idx,
@@ -453,47 +514,47 @@ class Grid:
             )
 
         # create TimeSeries
-        ops.timeSeries("Constant", 1)
+        osp.timeSeries("Constant", 1)
 
         # create a plain load pattern
-        ops.pattern("Plain", 1, 1)
+        osp.pattern("Plain", 1, 1)
 
         # Create the nodal load: nodeID loadvals
         # xForce yForce zForce xMoment yMoment zMoment
         for n in self.nodes:
             if n.Fz != 0 or n.Mx != 0 or n.My != 0:
                 loadVals = [0, 0, n.Fz, n.Mx, n.My, 0]
-                ops.load(n.idx, *loadVals)
+                osp.load(n.idx, *loadVals)
 
         # ------------------------------
         # Start of analysis generation
         # ------------------------------
 
         # create SOE
-        ops.system("FullGeneral")
+        osp.system("FullGeneral")
 
         # create DOF number
-        ops.numberer("RCM")
+        osp.numberer("RCM")
 
         # create constraint handler
-        ops.constraints("Plain")
+        osp.constraints("Plain")
 
         # create integrator
-        ops.integrator("LoadControl", 1.0)
+        osp.integrator("LoadControl", 1.0)
 
         # create algorithm
-        ops.algorithm("Linear")
+        osp.algorithm("Linear")
 
         # create analysis object
-        ops.analysis("Static")
+        osp.analysis("Static")
 
         # perform the analysis
-        ops.analyze(1)
+        osp.analyze(1)
 
         # calculate reactions
-        ops.reactions()
+        osp.reactions()
 
-        return ops
+        return osp
 
     def get_displacement(self, node=Union[Node, str, int], dof: int = -1):
         """
@@ -517,8 +578,8 @@ class Grid:
         None.
 
         """
-        the_node = self._get_node(node)
-        return ops.nodeDisp(the_node.idx, dof)
+        the_node = self.get_node(node)
+        return osp.nodeDisp(the_node.idx, dof)
 
     def get_reactions(self, node=Union[Node, str, int], dof: int = -1):
         """
@@ -542,8 +603,8 @@ class Grid:
         None.
 
         """
-        the_node = self._get_node(node)
-        return ops.nodeReaction(the_node.idx, dof)
+        the_node = self.get_node(node)
+        return osp.nodeReaction(the_node.idx, dof)
 
     def get_system_stiffness(self) -> np.ndarray:
         """
@@ -556,8 +617,8 @@ class Grid:
             A square numpy array of the reduced global stiffness matrix
 
         """
-        K = ops.printA("-ret")
-        n = ops.systemSize()
+        K = osp.printA("-ret")
+        n = osp.systemSize()
         return np.reshape(K, (n, n))
 
     def get_system_force(self) -> np.ndarray:
@@ -578,7 +639,8 @@ class Grid:
         self, member: Union[Member, int, Tuple[str, str]], dof: int = -1
     ) -> List[float]:
         """
-        Returns the member end force for the indicated DOF.
+        Returns the member end forces for the indicated DOF in the global coordinate
+        system.
 
         Parameters
         ----------
@@ -589,35 +651,13 @@ class Grid:
 
         Returns
         -------
-        F : List[float]
+        F : np.array
             A 12 element list: 6 DOFs for node i, and 6 DOFs for node j, in the order
             Fx, Fy, Fz, Mx, My, Mz.
 
         """
-        the_member = self._get_member(member)
-        return ops.eleForce(the_member.idx, dof)
-
-    def get_member_stiffness(
-        self, member: Union[Member, int, Tuple[str, str]]
-    ) -> np.ndarray:
-        """
-        Returns the member stiffness matrix in local coordinates with nodal DOFs
-        in the order DZ ('vertical' force), RX (torsion), RZ (bending moment).
-
-        Parameters
-        ----------
-        member : Union[Member, int, Tuple[str, str]]
-            Member object, id, or a tuple of node labels.
-
-        Returns
-        -------
-        K : np.ndarray
-            Member stiffness matrix in local coordinates.
-
-        """
-
-        the_member = self._get_member(member)
-        return the_member.get_local_stiffness()
+        the_member = self.get_member(member)
+        return np.array(osp.eleForce(the_member.idx, dof))
 
     def plot_results(
         self,
@@ -678,7 +718,7 @@ class Grid:
         if figsize is None:
             figsize = self.FIGSIZE
 
-        opsv.plot_model(fig_wi_he=figsize)
+        ospv.plot_model(fig_wi_he=figsize)
         fig = plt.gcf()
         fig.set_figwidth(figsize[0])
         fig.set_figheight(figsize[1])
@@ -710,28 +750,26 @@ class Grid:
         if figsize is None:
             figsize = self.FIGSIZE
 
-        opsv.plot_defo(sfac=scale_factor, endDispFlag=1)
+        ospv.plot_defo(sfac=scale_factor, endDispFlag=1)
         fig = plt.gcf()
         fig.set_figwidth(figsize[0])
         fig.set_figheight(figsize[1])
-        plt.gcf().suptitle("Displaced Shape")
+        plt.gcf().suptitle(f"Displaced Shape\n(Scale: {scale_factor})")
         if not axes_on:
             plt.gca().set_axis_off()
         else:
             fig.tight_layout()
 
-    def plot_bmd(
-        self, scale_factor: float = -1.0e-3, figsize=None, axes_on: bool = True
-    ):
+    def plot_bmd(self, scale_factor: float = 1.0, figsize=None, axes_on: bool = True):
         """
         Plot the bending moment diagram.
 
         Parameters
         ----------
         scale_factor : float
-            The scale of the bending moment to use. The default is -1e3. The negative
-            sign means to flip the diagram so that it appears on the tension face,
-            per convention.
+            The scale of the bending moment to use. The default is 1.0. A negative
+            sign is then applied to flip the diagram so that it appears on the tension
+            face, per convention.
         figsize : TYPE, optional
             The size of the figure in inches. The default is self.FIGSIZE.
         axes_on : bool, optional
@@ -745,27 +783,26 @@ class Grid:
         if figsize is None:
             figsize = self.FIGSIZE
 
-        opsv.section_force_diagram_3d("My", Ew={}, sfac=scale_factor)
+        ospv.section_force_diagram_3d("My", Ew={}, sfac=-scale_factor)
         fig = plt.gcf()
         fig.set_figwidth(figsize[0])
         fig.set_figheight(figsize[1])
-        fig.suptitle("Bending Moment Diagram")
+        fig.suptitle(f"Bending Moment Diagram\n(Scale: {scale_factor})")
 
         if not axes_on:
             plt.gca().set_axis_off()
         else:
             fig.tight_layout()
 
-    def plot_sfd(
-        self, scale_factor: float = 1.0e-3, figsize=None, axes_on: bool = True
-    ):
+    def plot_sfd(self, scale_factor: float = 1.0, figsize=None, axes_on: bool = True):
         """
         Plot the shear force diagram.
 
         Parameters
         ----------
         scale_factor : float
-            The scale of the bending moment to use. The default is -1e3.
+            The scale of the bending moment to use. The default is 1.0. A negative
+            sign is then applied to flip the diagram so that it appears per convention.
         figsize : TYPE, optional
             The size of the figure in inches. The default is self.FIGSIZE.
         axes_on : bool, optional
@@ -779,29 +816,26 @@ class Grid:
         if figsize is None:
             figsize = self.FIGSIZE
 
-        opsv.section_force_diagram_3d("Vz", Ew={}, sfac=scale_factor)
+        ospv.section_force_diagram_3d("Vz", Ew={}, sfac=-scale_factor)
         fig = plt.gcf()
         fig.set_figwidth(figsize[0])
         fig.set_figheight(figsize[1])
-        plt.gcf().suptitle("Shear Force Diagram")
+        plt.gcf().suptitle(f"Shear Force Diagram\n(Scale: {scale_factor})")
 
         if not axes_on:
             plt.gca().set_axis_off()
         else:
             fig.tight_layout()
 
-    def plot_tmd(
-        self, scale_factor: float = 1.0e-3, figsize=None, axes_on: bool = True
-    ):
+    def plot_tmd(self, scale_factor: float = 1.0, figsize=None, axes_on: bool = True):
         """
         Plot the torsion moment diagram.
 
         Parameters
         ----------
         scale_factor : float
-            The scale of the bending moment to use. The default is -1e3. The negative
-            sign means to flip the diagram so that it appears on the tension face,
-            per convention.
+            The scale of the bending moment to use. The default is 1.0.  A negative
+            sign is then applied to flip the diagram so that it appears per convention.
         figsize : TYPE, optional
             The size of the figure in inches. The default is self.FIGSIZE.
         axes_on : bool, optional
@@ -815,13 +849,137 @@ class Grid:
         if figsize is None:
             figsize = self.FIGSIZE
 
-        opsv.section_force_diagram_3d("T", Ew={}, sfac=scale_factor, dir_plt=2)
+        ospv.section_force_diagram_3d("T", Ew={}, sfac=-scale_factor, dir_plt=2)
         fig = plt.gcf()
         fig.set_figwidth(figsize[0])
         fig.set_figheight(figsize[1])
-        plt.gcf().suptitle("Torsion Moment Diagram")
+        plt.gcf().suptitle(f"Torsion Moment Diagram\n(Scale: {scale_factor})")
 
         if not axes_on:
             plt.gca().set_axis_off()
         else:
             fig.tight_layout()
+
+
+def make_grid(grid_str: str) -> Grid:
+    r"""
+    This function creates a grid object from a grid string specification. This
+    functionality is particularly useful for the bulk generation of grids, and the
+    grid specification is useful for the easy storage or communication of a grid.
+
+    .. note:: The grid is taken as lying in the `x`-`y` plane.
+
+    At a high level, the grid string specification has three parts, and looks as
+    follows:
+
+        ``"N_L_M"``
+
+    where ``N`` is a string specifying the nodes; ``L`` specifies
+    the nodal loads, and; ``M`` specifies the member connectivity. These parts are
+    described next.
+
+    The **node string** comprises a comma-separated list of sub-strings, each describing
+    its node as follows:
+
+        ``LSX:Y``
+
+    in which ``L`` is the node identifying letter (e.g. "A"); ``S`` is the nodal condition
+    which is one of "N" for no support, or one of the single character support descriptors
+    acceptable by :py:meth:`~.Grid.add_support`; ``X`` and ``Y`` define the `(x,y)`
+    coordinates of the node (note the separating colon ":").
+
+    The **load string** comprises a comma-separated list of substrings, each describing
+    the nodal loads as follows:
+
+        ``LFx:Mx:My``
+
+    in which ``L`` is the node identifier to which the loads are applied; ``Fx``, ``Mx``,
+    and ``My`` are the nodal loads as per :py:meth:`~.Grid.add_load`. Note the
+    separating colons between the nodal load components and that all 3 components
+    must be defined (even if zero "0"). Nodes with no loads applied do not need to
+    be included here.
+
+    The **member connectivity** substring describes the members, and comprises a
+    comma-separated list of two character nodal identifiers, the *i*- and *j*-node of
+    each member following :py:meth:`~.Grid.add_member`. These characters must be
+    from the list passed in the *node string*.
+
+    For clarity, the hierarchy of separators is:
+
+        - ``"_"`` separates the node, load, and member connectivity sub-strings;
+        - ``","`` separates entries in the lists for each of the node, load, and member sub-strings;
+        - ``":"`` separates coordinates or load components in a node or load definition.
+
+
+    **Example**::
+
+        import ospgrid as ospg
+
+        grid_str = "AF-5:0,BX0:-5,CN0:0,DY2:0_C0:-50:-50_AC,CD,BC"
+        grid = ospg.make_grid(grid_str)
+        grid.analyze()
+        grid.plot_results()
+
+    This is a 4-node, 3-member grid. Node `A` is at `(-5,0)` and has a :attr:`~.Support.FIXED`
+    support (``AF-5:0``); `B` is at `(0,-5)` and has a :attr:`~.Support.PINNED_X` support
+    (``BX0:-5``); `C` is at the origin and is unsupported (``CN0:0``); while `D` is at
+    `(2,0)` and has a :attr:`~.Support.PINNED_Y` support (``DY2:0``). The grid is
+    loaded at node `C` with :math:`M_x = -50` kNm and :math:`M_y = -50` kNm (``C0:-10:-10``)
+    and the members connect as follows: `AC`, `CD`, and `BC` (``AC,CD,BC``).
+
+    .. note::
+        This function assigns :math:`EI = 10\times10^3` kNm\ :sup:`2`, and :math:`GJ = 5\times10^3`
+        kNm\ :sup:`2`. This should be reasonable for member lengths <10 m and loads of the order
+        <10\ :sup:`2` kN or kNm.
+
+
+    Parameters
+    ----------
+    grid_str : str
+        The string describing the grid, formatted as described here.
+
+    Raises
+    ------
+    ValueError
+        If the string contains inconsistent information.
+
+    Returns
+    -------
+    Grid
+        The ospgrid object.
+
+    """
+    (nodes, loads, mbrs) = grid_str.split("_")
+
+    grid = Grid()
+    EI = 10e3  # kNm2
+    GJ = 5e3
+
+    node = nodes.split(",")
+    for n in node:
+        letter = n[0]
+        cond = n[1]
+        coords = n[2:].split(":")
+
+        grid.add_node(letter, float(coords[0]), float(coords[1]))
+        if cond != "N":  # N = None support
+            grid.add_support(letter, cond)
+
+    mbr = mbrs.split(",")
+    for m in mbr:
+        try:
+            grid.add_member(m[0], m[1], EI, GJ)
+        except Exception:
+            raise ValueError(f"Node letters {m}, member definition error")
+
+    load = loads.split(",")
+    for ld in load:
+        letter = ld[0]
+        vals = ld[1:].split(":")
+
+        if len(vals) != 3:
+            raise ValueError("Insufficient nodal load values")
+
+        grid.add_load(letter, Fz=float(vals[0]), Mx=float(vals[1]), My=float(vals[2]))
+
+    return grid
